package si.mojeznanje.model;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;

import javax.persistence.*;
import javax.xml.bind.annotation.XmlRootElement;
import java.time.ZoneId;
import java.util.*;

/**
 * Created by Homez on 16/01/2018.
 */
@JsonAutoDetect
@JsonIgnoreProperties(ignoreUnknown = true)
@XmlRootElement(name = "ograda")
@Entity
@Table(name = "ograda")
public class OgradaSet {

    private Integer id;
    private String ime = "ograda";
    private Set<Zival> zivali = new HashSet<>();
    private Set<Pes> psi = new HashSet<Pes>();
    private Set<Macka> macke = new HashSet<Macka>();
    private Set<Riba> ribe = new HashSet<Riba>();
    private Lokacija lokacija = null;

    private HashMap<String, Object> upravljalec = new HashMap<String, Object>();

    private Date datumOdprtja;

    public OgradaSet() {
        datumOdprtja = new Date();
    }


    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id", unique = true, nullable = false)
    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    @OneToOne(mappedBy = "ograda", cascade = CascadeType.ALL)
    public Lokacija getLokacija() {
        return lokacija;
    }

    public void setLokacija(Lokacija lokacija) {
        this.lokacija = lokacija;
    }

    @Column(name = "ime")
    public String getIme() {
        return ime;
    }

    public void setIme(String ime) {
        this.ime = ime;
    }

    @Transient
    public HashMap<String, Object> getUpravljalec(){
        return upravljalec;
    }

    public void setUpravljalec(HashMap<String, Object> upravljalec) {
        this.upravljalec = upravljalec;
    }

    @Transient
    public Date getDatumOdprtja() {
        return datumOdprtja;
    }

    public void setZivali(Set<Zival> zivali) {
        this.zivali = zivali;
    }


    @OneToMany(cascade = CascadeType.ALL)
    @JoinColumn(name = "ograda_id")
    public Set<Pes> getPsi() {
        return psi;
    }

    public void setPsi(Set<Pes> psi) {
        this.psi = psi;

        this.zivali.addAll(psi);
    }

    @OneToMany(cascade = CascadeType.ALL)
    @JoinColumn(name = "ograda_id")
    public Set<Macka> getMacke() {
        return macke;
    }

    public void setMacke(Set<Macka> macke) {
        this.macke = macke;
        this.zivali.addAll(macke);
    }

    @OneToMany(cascade = CascadeType.ALL)
    @JoinColumn(name = "ograda_id")
    public Set<Riba> getRibe() {
        return ribe;
    }

    public void setRibe(Set<Riba> ribe) {
        this.ribe = ribe;

        this.zivali.addAll(ribe);
    }

    public void setDatumOdprtja(Date datumOdprtja) {
        this.datumOdprtja = datumOdprtja;
    }

    @Transient
    @JsonIgnore
    public Set<Zival> getZivali(){
        return zivali;
    }

    public void dodajZival(Zival novaZival) {

        this.zivali.add(novaZival);

            if (novaZival instanceof Pes)
                psi.add((Pes)novaZival);
            else if(novaZival instanceof Macka)
                macke.add((Macka)novaZival);
            else if (novaZival instanceof Riba)
                ribe.add((Riba)novaZival);

    }

    public void dodajZivali(List<Zival> zivali) {

        for (Zival novaZival : zivali)
        {

            dodajZival(novaZival);
        }

    }

    public void izipisiVseZivali() {
        System.out.println("V ogradi je " + zivali.size() + " zivali.");

        if (zivali.size() > 0) {

            for (Zival zival : zivali) {
                System.out.println(zival.toString());
            }
        }
    }

    public Float vrniPovprecnoStarostZivaliVOgradi()
    {

      List<Integer> starosti = new ArrayList<Integer>();

      for(Zival zival : zivali){
          starosti.add(zival.getStarost());
      }
        Float povprecnaStarost = OgradaSet.izracunajPovprecnoStarost(starosti);

        return povprecnaStarost;

    }

    public static Float izracunajPovprecnoStarost(List<Integer> starosti){

        Float povprecnaStarost = 0f;
        Float skupekStarosti = 0f;

        for(int i = 0; i < starosti.size(); i++){
            Integer starost = starosti.get(i);
            skupekStarosti = skupekStarosti + starost;
        }

        povprecnaStarost = skupekStarosti / starosti.size();

        return povprecnaStarost;
    }

    public static Float izracunajPovprecnoStarost(Integer[] starosti){

        Float povprecnaStarost = 0f;
        Float skupekStarosti = 0f;

        for(int i = 0; i < starosti.length; i++){
            Integer starost = starosti[i];
            skupekStarosti = skupekStarosti + starost;
        }

        povprecnaStarost = skupekStarosti / starosti.length;

        return povprecnaStarost;
    }
}
