package si.mojeznanje.model;


import javax.persistence.*;
import java.util.Objects;

@MappedSuperclass
public abstract class Zival  implements Comparable<Zival> {

    public static final Integer MOSKI = 0;
    public static final Integer ZENSKI = 1;

    private Integer id;
    private String ime;
    private Integer starost;
    private Integer spol;

    public Zival(){
    }

    public Zival(String ime, Integer starost){
        this.ime = ime;
        this.starost = starost;
    }

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id", unique = true, nullable = false)
    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    @Column(name = "ime")
    public String getIme() {
        return ime;
    }

    public void setIme(String ime) {

        if(ime == null)
            ime = "DEFAULT";

        if(ime == "")
            ime = "DEFAULT";

        this.ime = ime;
    }


    @Column(name = "starost")
    public Integer getStarost() {
        return starost;
    }

    public void setStarost(Integer starost) {

        if(starost < 0)
            starost = 0;

        this.starost = starost;
    }


    @Column(name = "spol")
    public Integer getSpol() {
        return spol;
    }

    public void setSpol(Integer spol) throws Exception {

        if(spol != MOSKI && spol != ZENSKI)
            throw new Exception("To ni validen spol!");

        this.spol = spol;
    }

    @Override
    public String toString() {
        return "ime: " + ime + " starost: " + starost + " spol : " + spol;
    }

    public Boolean equals(Zival zival){

        if(zival == null){
            return false;
        }

        if(!this.getClass().equals(zival.getClass())){
            return false;
        }

        return this.getIme().equals(zival.getIme()) && this.getStarost() == zival.getStarost() && this.getSpol() == zival.getSpol();
    }

    public abstract void oglasiSe();

    public String toCSV() {
        return getClass().getCanonicalName()+"," + getIme() + "," + getStarost() + "," + getSpol();
    }

    @Override
    public int hashCode(){
        return Objects.hash(this.ime, this.starost, this.spol, this.getClass().getCanonicalName());
    }

    @Override
    public boolean equals(Object object){

        if(!(object instanceof Zival))
            return false;

        Zival zival = (Zival)object;

        if(zival == null){
            return false;
        }

        if(!this.getClass().equals(zival.getClass())){
            return false;
        }

        return this.getIme().equals(zival.getIme()) && this.getStarost() == zival.getStarost() && this.getSpol() == zival.getSpol();
    }

    @Override
    public int compareTo(Zival zival){

        return getStarost().compareTo(zival.getStarost());

        //this je manj (prej) od parametra : -1
        //this je enako kot parameter : 0
        //this je več od paramtra : 1
    }
}
